/****************************************************************************
** Meta object code from reading C++ file 'applicationmanager1service.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/dbus/applicationmanager1service.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'applicationmanager1service.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17UnitResultWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto UnitResultWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN17UnitResultWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "UnitResultWatcher",
        "resultReady",
        "",
        "QDBusObjectPath",
        "unitPath",
        "result",
        "handlePropertyChanged",
        "interface",
        "QVariantMap",
        "changed",
        "invalidated"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resultReady'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'handlePropertyChanged'
        QtMocHelpers::SlotData<void(const QString &, const QVariantMap &, const QStringList &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 }, { QMetaType::QStringList, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<UnitResultWatcher, qt_meta_tag_ZN17UnitResultWatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject UnitResultWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17UnitResultWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17UnitResultWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17UnitResultWatcherE_t>.metaTypes,
    nullptr
} };

void UnitResultWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<UnitResultWatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resultReady((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->handlePropertyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (UnitResultWatcher::*)(const QDBusObjectPath & , const QString & )>(_a, &UnitResultWatcher::resultReady, 0))
            return;
    }
}

const QMetaObject *UnitResultWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UnitResultWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17UnitResultWatcherE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int UnitResultWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void UnitResultWatcher::resultReady(const QDBusObjectPath & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN26ApplicationManager1ServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto ApplicationManager1Service::qt_create_metaobjectdata<qt_meta_tag_ZN26ApplicationManager1ServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ApplicationManager1Service",
        "InterfacesAdded",
        "",
        "QDBusObjectPath",
        "object_path",
        "ObjectInterfaceMap",
        "interfaces",
        "InterfacesRemoved",
        "listChanged",
        "executeCommand",
        "program",
        "arguments",
        "type",
        "runId",
        "QMap<QString,QString>",
        "envVars",
        "workdir",
        "Identify",
        "QDBusUnixFileDescriptor",
        "pidfd",
        "QDBusObjectPath&",
        "instance",
        "ObjectInterfaceMap&",
        "application_instance_info",
        "ReloadApplications",
        "addUserApplication",
        "QVariantMap",
        "desktop_file",
        "name",
        "deleteUserApplication",
        "app_id",
        "GetManagedObjects",
        "ObjectMap",
        "doReloadApplications",
        "List",
        "QList<QDBusObjectPath>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'InterfacesAdded'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const ObjectInterfaceMap &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'InterfacesRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QStringList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QStringList, 6 },
        }}),
        // Signal 'listChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'executeCommand'
        QtMocHelpers::SlotData<QDBusObjectPath(const QString &, const QStringList &, const QString &, const QString &, const QMap<QString,QString> &, const QString &)>(9, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 10 }, { QMetaType::QStringList, 11 }, { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
            { 0x80000000 | 14, 15 }, { QMetaType::QString, 16 },
        }}),
        // Slot 'Identify'
        QtMocHelpers::SlotData<QString(const QDBusUnixFileDescriptor &, QDBusObjectPath &, ObjectInterfaceMap &) const>(17, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'ReloadApplications'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addUserApplication'
        QtMocHelpers::SlotData<QString(const QVariantMap &, const QString &)>(25, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 26, 27 }, { QMetaType::QString, 28 },
        }}),
        // Slot 'deleteUserApplication'
        QtMocHelpers::SlotData<void(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Slot 'GetManagedObjects'
        QtMocHelpers::SlotData<ObjectMap() const>(31, 2, QMC::AccessPublic, 0x80000000 | 32),
        // Slot 'doReloadApplications'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'List'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(34, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ApplicationManager1Service, qt_meta_tag_ZN26ApplicationManager1ServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ApplicationManager1Service::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26ApplicationManager1ServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26ApplicationManager1ServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26ApplicationManager1ServiceE_t>.metaTypes,
    nullptr
} };

void ApplicationManager1Service::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ApplicationManager1Service *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->InterfacesAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectInterfaceMap>>(_a[2]))); break;
        case 1: _t->InterfacesRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 2: _t->listChanged(); break;
        case 3: { QDBusObjectPath _r = _t->executeCommand((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QString>>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])));
            if (_a[0]) *reinterpret_cast<QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->Identify((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<ObjectInterfaceMap&>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->ReloadApplications(); break;
        case 6: { QString _r = _t->addUserApplication((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->deleteUserApplication((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: { ObjectMap _r = _t->GetManagedObjects();
            if (_a[0]) *reinterpret_cast<ObjectMap*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->doReloadApplications(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ObjectInterfaceMap >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusUnixFileDescriptor >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ApplicationManager1Service::*)(const QDBusObjectPath & , const ObjectInterfaceMap & )>(_a, &ApplicationManager1Service::InterfacesAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationManager1Service::*)(const QDBusObjectPath & , const QStringList & )>(_a, &ApplicationManager1Service::InterfacesRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationManager1Service::*)()>(_a, &ApplicationManager1Service::listChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->list(); break;
        default: break;
        }
    }
}

const QMetaObject *ApplicationManager1Service::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ApplicationManager1Service::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26ApplicationManager1ServiceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int ApplicationManager1Service::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void ApplicationManager1Service::InterfacesAdded(const QDBusObjectPath & _t1, const ObjectInterfaceMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void ApplicationManager1Service::InterfacesRemoved(const QDBusObjectPath & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void ApplicationManager1Service::listChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
