/****************************************************************************
** Meta object code from reading C++ file 'mimemanager1adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "mimemanager1adaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mimemanager1adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19MimeManager1AdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto MimeManager1Adaptor::qt_create_metaobjectdata<qt_meta_tag_ZN19MimeManager1AdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MimeManager1Adaptor",
        "D-Bus Interface",
        "org.desktopspec.MimeManager1",
        "D-Bus Introspection",
        "  <interface name=\"org.desktopspec.MimeManager1\">\n    <method n"
        "ame=\"queryDefaultApplication\">\n      <arg direction=\"in\" type"
        "=\"s\" name=\"content\"/>\n      <arg direction=\"out\" type=\"s\""
        " name=\"mimeType\"/>\n      <arg direction=\"out\" type=\"o\" name"
        "=\"application\"/>\n      <annotation value=\"content can be absol"
        "ute path of a file or a mime type.\" name=\"org.freedesktop.DBus.D"
        "escription\"/>\n    </method>\n    <method name=\"setDefaultApplic"
        "ation\">\n      <arg direction=\"in\" type=\"a{ss}\" name=\"defaul"
        "tApps\"/>\n      <annotation value=\"QStringMap\" name=\"org.qtpro"
        "ject.QtDBus.QtTypeName.In0\"/>\n    </method>\n    <method name=\""
        "unsetDefaultApplication\">\n      <arg direction=\"in\" type=\"as\""
        " name=\"mimeTypes\"/>\n    </method>\n    <method name=\"listAppli"
        "cations\">\n      <arg direction=\"in\" type=\"s\" name=\"mimeType"
        "\"/>\n      <arg direction=\"out\" type=\"a{oa{sa{sv}}}\" name=\"a"
        "pplications_and_properties\"/>\n      <annotation value=\"ObjectMa"
        "p\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n    </method>"
        "\n    <signal name=\"MimeInfoReloaded\">\n      <annotation value="
        "\"Signal emitted when the mime infos are reloaded.\" name=\"org.fr"
        "eedesktop.DBus.Description\"/>\n    </signal>\n  </interface>\n",
        "MimeInfoReloaded",
        "",
        "listApplications",
        "ObjectMap",
        "mimeType",
        "queryDefaultApplication",
        "content",
        "QDBusObjectPath&",
        "application",
        "setDefaultApplication",
        "QStringMap",
        "defaultApps",
        "unsetDefaultApplication",
        "mimeTypes"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'MimeInfoReloaded'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'listApplications'
        QtMocHelpers::SlotData<ObjectMap(const QString &)>(7, 6, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 9 },
        }}),
        // Slot 'queryDefaultApplication'
        QtMocHelpers::SlotData<QString(const QString &, QDBusObjectPath &)>(10, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'setDefaultApplication'
        QtMocHelpers::SlotData<void(const QStringMap &)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'unsetDefaultApplication'
        QtMocHelpers::SlotData<void(const QStringList &)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<MimeManager1Adaptor, qt_meta_tag_ZN19MimeManager1AdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject MimeManager1Adaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19MimeManager1AdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19MimeManager1AdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19MimeManager1AdaptorE_t>.metaTypes,
    nullptr
} };

void MimeManager1Adaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MimeManager1Adaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->MimeInfoReloaded(); break;
        case 1: { ObjectMap _r = _t->listApplications((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<ObjectMap*>(_a[0]) = std::move(_r); }  break;
        case 2: { QString _r = _t->queryDefaultApplication((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath&>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->setDefaultApplication((*reinterpret_cast<std::add_pointer_t<QStringMap>>(_a[1]))); break;
        case 4: _t->unsetDefaultApplication((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MimeManager1Adaptor::*)()>(_a, &MimeManager1Adaptor::MimeInfoReloaded, 0))
            return;
    }
}

const QMetaObject *MimeManager1Adaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MimeManager1Adaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19MimeManager1AdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int MimeManager1Adaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void MimeManager1Adaptor::MimeInfoReloaded()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
