// This file was generated by qtwaylandscanner
// source file is treeland-prelaunch-splash-v2.xml

#ifndef QT_WAYLAND_TREELAND_PRELAUNCH_SPLASH_V2
#define QT_WAYLAND_TREELAND_PRELAUNCH_SPLASH_V2

#include "wayland-treeland-prelaunch-splash-v2-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  treeland_prelaunch_splash_manager_v2
    {
    public:
        treeland_prelaunch_splash_manager_v2(struct ::wl_registry *registry, uint32_t id, int version);
        treeland_prelaunch_splash_manager_v2(struct ::treeland_prelaunch_splash_manager_v2 *object);
        treeland_prelaunch_splash_manager_v2();

        virtual ~treeland_prelaunch_splash_manager_v2();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::treeland_prelaunch_splash_manager_v2 *object);

        struct ::treeland_prelaunch_splash_manager_v2 *object() { return m_treeland_prelaunch_splash_manager_v2; }
        const struct ::treeland_prelaunch_splash_manager_v2 *object() const { return m_treeland_prelaunch_splash_manager_v2; }
        static treeland_prelaunch_splash_manager_v2 *fromObject(struct ::treeland_prelaunch_splash_manager_v2 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void destroy();
        struct ::treeland_prelaunch_splash_v2 *create_splash(const QString &app_id, const QString &instance_id, const QString &sandbox_engine_name, struct ::wl_buffer *icon_buffer);

    private:
        struct ::treeland_prelaunch_splash_manager_v2 *m_treeland_prelaunch_splash_manager_v2;
    };

    class  treeland_prelaunch_splash_v2
    {
    public:
        treeland_prelaunch_splash_v2(struct ::wl_registry *registry, uint32_t id, int version);
        treeland_prelaunch_splash_v2(struct ::treeland_prelaunch_splash_v2 *object);
        treeland_prelaunch_splash_v2();

        virtual ~treeland_prelaunch_splash_v2();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::treeland_prelaunch_splash_v2 *object);

        struct ::treeland_prelaunch_splash_v2 *object() { return m_treeland_prelaunch_splash_v2; }
        const struct ::treeland_prelaunch_splash_v2 *object() const { return m_treeland_prelaunch_splash_v2; }
        static treeland_prelaunch_splash_v2 *fromObject(struct ::treeland_prelaunch_splash_v2 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void destroy();

    private:
        struct ::treeland_prelaunch_splash_v2 *m_treeland_prelaunch_splash_v2;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
