/****************************************************************************
** Meta object code from reading C++ file 'jobmanager1adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "jobmanager1adaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'jobmanager1adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18JobManager1AdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto JobManager1Adaptor::qt_create_metaobjectdata<qt_meta_tag_ZN18JobManager1AdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JobManager1Adaptor",
        "D-Bus Interface",
        "org.desktopspec.JobManager1",
        "D-Bus Introspection",
        "  <interface name=\"org.desktopspec.JobManager1\">\n    <annotatio"
        "n value=\"All method might block will return a DBus object path,  "
        "                  which implement org.desktopspec.JobManager1.Job."
        "                    This interface is design to provide some DBus "
        "signals to help                    caller to watch the Jobs they i"
        "nvoked.                    Caller should not interst in all the Jo"
        "bs, as there are some                    Jobs not created by them."
        "                    So the method to list all exsiting Jobs is NOT"
        " provided.                    NOTE:                    Signal emit"
        "ted by this interface MIGHT be peer-to-peer.\" name=\"org.freedesk"
        "top.DBus.Description\"/>\n    <signal name=\"JobNew\">\n      <arg"
        " type=\"o\" name=\"job\"/>\n      <arg type=\"o\" name=\"source\"/"
        ">\n      <annotation value=\"`source` is the DBus object produce t"
        "his job.\" name=\"org.freedesktop.DBus.Description\"/>\n    </sign"
        "al>\n    <signal name=\"JobRemoved\">\n      <arg type=\"o\" name="
        "\"job\"/>\n      <arg type=\"s\" name=\"status\"/>\n      <arg typ"
        "e=\"av\" name=\"result\"/>\n      <annotation value=\"This signal "
        "report the job remove event.                        `status` is th"
        "e `status` porperty of that Job                        when it got"
        " removed.                        `result` is the result which prod"
        "uce by this job,                        and user needs to traverse"
        " the list to determine                         if the values are v"
        "alid or not.                        Method that return an object o"
        "f Job                        should always have document          "
        "              about the type and meaning of the `result`.\" name=\""
        "org.freedesktop.DBus.Description\"/>\n    </signal>\n  </interface"
        ">\n",
        "JobNew",
        "",
        "QDBusObjectPath",
        "job",
        "source",
        "JobRemoved",
        "status",
        "QVariantList",
        "result"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'JobNew'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QDBusObjectPath &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 7, 9 },
        }}),
        // Signal 'JobRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QString &, const QVariantList &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { QMetaType::QString, 11 }, { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<JobManager1Adaptor, qt_meta_tag_ZN18JobManager1AdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject JobManager1Adaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JobManager1AdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JobManager1AdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18JobManager1AdaptorE_t>.metaTypes,
    nullptr
} };

void JobManager1Adaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JobManager1Adaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->JobNew((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 1: _t->JobRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (JobManager1Adaptor::*)(const QDBusObjectPath & , const QDBusObjectPath & )>(_a, &JobManager1Adaptor::JobNew, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (JobManager1Adaptor::*)(const QDBusObjectPath & , const QString & , const QVariantList & )>(_a, &JobManager1Adaptor::JobRemoved, 1))
            return;
    }
}

const QMetaObject *JobManager1Adaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JobManager1Adaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18JobManager1AdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int JobManager1Adaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void JobManager1Adaptor::JobNew(const QDBusObjectPath & _t1, const QDBusObjectPath & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void JobManager1Adaptor::JobRemoved(const QDBusObjectPath & _t1, const QString & _t2, const QVariantList & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
