/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.ApplicationUpdateNotifier1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef APPLICATIONUPDATENOTIFIER1ADAPTOR_H
#define APPLICATIONUPDATENOTIFIER1ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "dbus/applicationupdatenotifier1service.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.desktopspec.ApplicationUpdateNotifier1
 */
class ApplicationUpdateNotifier1Adaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.desktopspec.ApplicationUpdateNotifier1")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.desktopspec.ApplicationUpdateNotifier1\">\n"
"    <signal name=\"ApplicationUpdated\">\n"
"      <annotation value=\"This signal will emit when application's                        infomations need to update.\" name=\"org.freedesktop.DBus.Description\"/>\n"
"    </signal>\n"
"  </interface>\n"
        "")
public:
    ApplicationUpdateNotifier1Adaptor(ApplicationUpdateNotifier1Service *parent);
    ~ApplicationUpdateNotifier1Adaptor() override;

    inline ApplicationUpdateNotifier1Service *parent() const
    { return static_cast<ApplicationUpdateNotifier1Service *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void ApplicationUpdated();
};

#endif
