/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.ApplicationManager1.Instance.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "instanceadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class InstanceAdaptor
 */

InstanceAdaptor::InstanceAdaptor(InstanceService *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

InstanceAdaptor::~InstanceAdaptor()
{
    // destructor
}

QDBusObjectPath InstanceAdaptor::application() const
{
    // get the value of property Application
    return qvariant_cast< QDBusObjectPath >(parent()->property("Application"));
}

QString InstanceAdaptor::launchType() const
{
    // get the value of property LaunchType
    return qvariant_cast< QString >(parent()->property("LaunchType"));
}

QString InstanceAdaptor::launcher() const
{
    // get the value of property Launcher
    return qvariant_cast< QString >(parent()->property("Launcher"));
}

bool InstanceAdaptor::orphaned() const
{
    // get the value of property Orphaned
    return qvariant_cast< bool >(parent()->property("Orphaned"));
}

QDBusObjectPath InstanceAdaptor::systemdUnitPath() const
{
    // get the value of property SystemdUnitPath
    return qvariant_cast< QDBusObjectPath >(parent()->property("SystemdUnitPath"));
}

void InstanceAdaptor::KillAll(int signal)
{
    // handle method call org.desktopspec.ApplicationManager1.Instance.KillAll
    parent()->KillAll(signal);
}


#include "moc_instanceadaptor.cpp"
