/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.ApplicationManager1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "applicationmanager1adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class ApplicationManager1Adaptor
 */

ApplicationManager1Adaptor::ApplicationManager1Adaptor(ApplicationManager1Service *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

ApplicationManager1Adaptor::~ApplicationManager1Adaptor()
{
    // destructor
}

QList<QDBusObjectPath> ApplicationManager1Adaptor::list() const
{
    // get the value of property List
    return qvariant_cast< QList<QDBusObjectPath> >(parent()->property("List"));
}

QString ApplicationManager1Adaptor::Identify(const QDBusUnixFileDescriptor &pidfd, QDBusObjectPath &instance, ObjectInterfaceMap &application_instance_info)
{
    // handle method call org.desktopspec.ApplicationManager1.Identify
    return parent()->Identify(pidfd, instance, application_instance_info);
}

void ApplicationManager1Adaptor::ReloadApplications()
{
    // handle method call org.desktopspec.ApplicationManager1.ReloadApplications
    parent()->ReloadApplications();
}

QString ApplicationManager1Adaptor::addUserApplication(const QVariantMap &desktop_file, const QString &name)
{
    // handle method call org.desktopspec.ApplicationManager1.addUserApplication
    return parent()->addUserApplication(desktop_file, name);
}

void ApplicationManager1Adaptor::deleteUserApplication(const QString &app_id)
{
    // handle method call org.desktopspec.ApplicationManager1.deleteUserApplication
    parent()->deleteUserApplication(app_id);
}

QDBusObjectPath ApplicationManager1Adaptor::executeCommand(const QString &program, const QStringList &arguments, const QString &type, const QString &runId, const QStringMap &envVars, const QString &workdir)
{
    // handle method call org.desktopspec.ApplicationManager1.executeCommand
    return parent()->executeCommand(program, arguments, type, runId, envVars, workdir);
}


#include "moc_applicationmanager1adaptor.cpp"
