/****************************************************************************
** Meta object code from reading C++ file 'applicationadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "applicationadaptor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'applicationadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18ApplicationAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto ApplicationAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN18ApplicationAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ApplicationAdaptor",
        "D-Bus Interface",
        "org.desktopspec.ApplicationManager1.Application",
        "D-Bus Introspection",
        "  <interface name=\"org.desktopspec.ApplicationManager1.Applicatio"
        "n\">\n    <annotation value=\"This interface is designed to provid"
        "e a dbus interface of desktop file. Missing fields will be added l"
        "ater.\" name=\"org.freedesktop.DBus.Description\"/>\n    <property"
        " access=\"read\" type=\"as\" name=\"Categories\"/>\n    <property "
        "access=\"read\" type=\"b\" name=\"X_linglong\"/>\n    <property ac"
        "cess=\"read\" type=\"b\" name=\"X_Flatpak\"/>\n    <property acces"
        "s=\"read\" type=\"s\" name=\"X_Deepin_Vendor\">\n      <annotation"
        " value=\"Whem this property is 'deepin', display name of the appli"
        "cation                               should use GenericName.\" nam"
        "e=\"org.freedesktop.DBus.Description\"/>\n    </property>\n    <pr"
        "operty access=\"read\" type=\"b\" name=\"NoDisplay\"/>\n    <prope"
        "rty access=\"readwrite\" type=\"as\" name=\"MimeTypes\"/>\n    <pr"
        "operty access=\"read\" type=\"as\" name=\"Actions\">\n      <annot"
        "ation value=\"Names of all action identifiers of this application."
        "                        Check https://specifications.freedesktop.o"
        "rg/desktop-entry-spec/desktop-entry-spec-latest.html#extra-actions"
        "                        for futher information.\" name=\"org.freed"
        "esktop.DBus.Description\"/>\n    </property>\n    <property access"
        "=\"readwrite\" type=\"b\" name=\"AutoStart\"/>\n    <property acce"
        "ss=\"read\" type=\"x\" name=\"LastLaunchedTime\">\n      <annotati"
        "on value=\"Set the value of this property to -1 to                "
        "        indicates that some errors has occured.\" name=\"org.freed"
        "esktop.DBus.Description\"/>\n    </property>\n    <property access"
        "=\"read\" type=\"x\" name=\"LaunchedTimes\">\n      <annotation va"
        "lue=\"Set the value of this property to -1 to                     "
        "   indicates that some errors has occured.\" name=\"org.freedeskto"
        "p.DBus.Description\"/>\n    </property>\n    <property access=\"re"
        "ad\" type=\"x\" name=\"InstalledTime\">\n      <annotation value=\""
        "Set the value of this property to -1 to                    indicat"
        "es that some errors has occured.\" name=\"org.freedesktop.DBus.Des"
        "cription\"/>\n    </property>\n    <property access=\"read\" type="
        "\"ao\" name=\"Instances\">\n      <annotation value=\"All instance"
        "s of this application.\" name=\"org.freedesktop.DBus.Description\""
        "/>\n    </property>\n    <property access=\"read\" type=\"s\" name"
        "=\"ID\">\n      <annotation value=\"The desktop file id of this ap"
        "plication.                        Check https://specifications.fre"
        "edesktop.org/desktop-entry-spec/desktop-entry-spec-latest.html#des"
        "ktop-file-id                        for futher infomation.\" name="
        "\"org.freedesktop.DBus.Description\"/>\n    </property>\n    <prop"
        "erty access=\"read\" type=\"b\" name=\"Terminal\">\n      <annotat"
        "ion value=\"Indicate this application should launch by terminal or"
        " not.\" name=\"org.freedesktop.DBus.Description\"/>\n    </propert"
        "y>\n    <property access=\"readwrite\" type=\"s\" name=\"Environ\""
        ">\n      <annotation value=\"Indicate the environment of applicati"
        "on's instance.                        passing some specific enviro"
        "nment variables to Launch                        this application,"
        " eg. 'LANG=en_US;PATH=xxx:yyy;'\" name=\"org.freedesktop.DBus.Desc"
        "ription\"/>\n    </property>\n    <property access=\"read\" type=\""
        "a{sa{ss}}\" name=\"ActionName\">\n      <annotation value=\"The ty"
        "pe of ActionName is a Map, first key represents action, second key"
        " represents locale and the value is the corresponding content.\" n"
        "ame=\"org.freedesktop.DBus.Description\"/>\n      <annotation valu"
        "e=\"PropMap\" name=\"org.qtproject.QtDBus.QtTypeName\"/>\n    </pr"
        "operty>\n    <property access=\"read\" type=\"a{ss}\" name=\"Icons"
        "\">\n      <annotation value=\"The type of IconName is a Map, wher"
        "e the key represents the action and the value is the corresponding"
        " content.\" name=\"org.freedesktop.DBus.Description\"/>\n      <an"
        "notation value=\"QStringMap\" name=\"org.qtproject.QtDBus.QtTypeNa"
        "me\"/>\n    </property>\n    <property access=\"read\" type=\"a{ss"
        "}\" name=\"Name\">\n      <annotation value=\"The meaning of this "
        "property's type is same as which in ActionName.\" name=\"org.freed"
        "esktop.DBus.Description\"/>\n      <annotation value=\"QStringMap\""
        " name=\"org.qtproject.QtDBus.QtTypeName\"/>\n    </property>\n    "
        "<property access=\"read\" type=\"a{ss}\" name=\"GenericName\">\n  "
        "    <annotation value=\"The meaning of this property's type is sam"
        "e as which in ActionName.\" name=\"org.freedesktop.DBus.Descriptio"
        "n\"/>\n      <annotation value=\"QStringMap\" name=\"org.qtproject"
        ".QtDBus.QtTypeName\"/>\n    </property>\n    <property access=\"re"
        "ad\" type=\"s\" name=\"StartupWMClass\">\n      <annotation value="
        "\"The meaning of this property's type is same as which in StartupW"
        "MClass.\" name=\"org.freedesktop.DBus.Description\"/>\n      <anno"
        "tation value=\"QStringMap\" name=\"org.qtproject.QtDBus.QtTypeName"
        "\"/>\n    </property>\n    <method name=\"Launch\">\n      <arg di"
        "rection=\"in\" type=\"s\" name=\"action\"/>\n      <arg direction="
        "\"in\" type=\"as\" name=\"fields\"/>\n      <arg direction=\"out\""
        " type=\"o\" name=\"job\"/>\n      <arg direction=\"in\" type=\"a{s"
        "v}\" name=\"options\"/>\n      <annotation value=\"QVariantMap\" n"
        "ame=\"org.qtproject.QtDBus.QtTypeName.In2\"/>\n      <annotation v"
        "alue=\"Given an action identifier,                        and some"
        " fields (file path or URI),                        this method wil"
        "l launch this application,                        and the object p"
        "ath of the job launching application.                        Resul"
        "t of that job is a object path                        of the new a"
        "pplication instance.                        If that job failed, th"
        "e result is a bool `false`.                         Extra options "
        "can be passed in `options` argument:                        1. `ui"
        "d` (type u):                           The user id as who is that "
        "application will be run.                           This option mig"
        "ht request a polikit authentication.                        2. `en"
        "v` (type as):                           passing some specific envi"
        "ronment variables to Launch                           this applica"
        "tion without them, eg. '[LANG=en_US, PATH=xxx:yyy]'               "
        "         3. `unsetEnv` (type as):                           passed"
        " environment variables will be ignored when                       "
        "    launching this application, eg. '[LANG, PATH]'                "
        "        4. `path` (type s):                           set this app"
        "lication's working directory, please pass                         "
        "  absolute directory path.                        NOTE:           "
        "             When application launched with `uid` option,         "
        "               `env` option will not take effect at all.          "
        "              The following internal options (prefixed with `_`)  "
        "                      are for internal use only and should not be "
        "used                        by external callers:                  "
        "      5. `_launch_type` (type s):                           launch"
        " source name for event reporting,                           e.g. '"
        "dde-launchpad', 'dde-shell'.                           Defaults to"
        " 'unknown' if not specified.\" name=\"org.freedesktop.DBus.Descrip"
        "tion\"/>\n    </method>\n    <property access=\"read\" type=\"b\" "
        "name=\"isOnDesktop\"/>\n    <property access=\"read\" type=\"s\" n"
        "ame=\"X_Deepin_CreateBy\"/>\n    <property access=\"read\" type=\""
        "a{ss}\" name=\"Execs\">\n      <annotation value=\"Display the exe"
        "c content of all actions\" name=\"org.freedesktop.DBus.Description"
        "\"/>\n      <annotation value=\"QStringMap\" name=\"org.qtproject."
        "QtDBus.QtTypeName\"/>\n    </property>\n    <property access=\"rea"
        "d\" type=\"s\" name=\"X_CreatedBy\"/>\n    <property access=\"read"
        "\" type=\"s\" name=\"DesktopSourcePath\"/>\n    <method name=\"Sen"
        "dToDesktop\">\n      <arg direction=\"out\" type=\"b\" name=\"succ"
        "ess\"/>\n    </method>\n    <method name=\"RemoveFromDesktop\">\n "
        "     <arg direction=\"out\" type=\"b\" name=\"success\"/>\n    </m"
        "ethod>\n  </interface>\n",
        "Launch",
        "QDBusObjectPath",
        "",
        "action",
        "fields",
        "QVariantMap",
        "options",
        "RemoveFromDesktop",
        "SendToDesktop",
        "ActionName",
        "PropMap",
        "Actions",
        "AutoStart",
        "Categories",
        "DesktopSourcePath",
        "Environ",
        "Execs",
        "QStringMap",
        "GenericName",
        "ID",
        "Icons",
        "InstalledTime",
        "Instances",
        "QList<QDBusObjectPath>",
        "LastLaunchedTime",
        "LaunchedTimes",
        "MimeTypes",
        "Name",
        "NoDisplay",
        "StartupWMClass",
        "Terminal",
        "X_CreatedBy",
        "X_Deepin_CreateBy",
        "X_Deepin_Vendor",
        "X_Flatpak",
        "X_linglong",
        "isOnDesktop"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'Launch'
        QtMocHelpers::SlotData<QDBusObjectPath(const QString &, const QStringList &, const QVariantMap &)>(5, 7, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 8 }, { QMetaType::QStringList, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'RemoveFromDesktop'
        QtMocHelpers::SlotData<bool()>(12, 7, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'SendToDesktop'
        QtMocHelpers::SlotData<bool()>(13, 7, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'ActionName'
        QtMocHelpers::PropertyData<PropMap>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'Actions'
        QtMocHelpers::PropertyData<QStringList>(16, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'AutoStart'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'Categories'
        QtMocHelpers::PropertyData<QStringList>(18, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'DesktopSourcePath'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Environ'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'Execs'
        QtMocHelpers::PropertyData<QStringMap>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'GenericName'
        QtMocHelpers::PropertyData<QStringMap>(23, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'ID'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Icons'
        QtMocHelpers::PropertyData<QStringMap>(25, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'InstalledTime'
        QtMocHelpers::PropertyData<qlonglong>(26, QMetaType::LongLong, QMC::DefaultPropertyFlags),
        // property 'Instances'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'LastLaunchedTime'
        QtMocHelpers::PropertyData<qlonglong>(29, QMetaType::LongLong, QMC::DefaultPropertyFlags),
        // property 'LaunchedTimes'
        QtMocHelpers::PropertyData<qlonglong>(30, QMetaType::LongLong, QMC::DefaultPropertyFlags),
        // property 'MimeTypes'
        QtMocHelpers::PropertyData<QStringList>(31, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'Name'
        QtMocHelpers::PropertyData<QStringMap>(32, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'NoDisplay'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'StartupWMClass'
        QtMocHelpers::PropertyData<QString>(34, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Terminal'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'X_CreatedBy'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'X_Deepin_CreateBy'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'X_Deepin_Vendor'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'X_Flatpak'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'X_linglong'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'isOnDesktop'
        QtMocHelpers::PropertyData<bool>(41, QMetaType::Bool, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<ApplicationAdaptor, qt_meta_tag_ZN18ApplicationAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ApplicationAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ApplicationAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ApplicationAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18ApplicationAdaptorE_t>.metaTypes,
    nullptr
} };

void ApplicationAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ApplicationAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDBusObjectPath _r = _t->Launch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->RemoveFromDesktop();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->SendToDesktop();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<PropMap*>(_v) = _t->actionName(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->actions(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->autoStart(); break;
        case 3: *reinterpret_cast<QStringList*>(_v) = _t->categories(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->desktopSourcePath(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->environ(); break;
        case 6: *reinterpret_cast<QStringMap*>(_v) = _t->execs(); break;
        case 7: *reinterpret_cast<QStringMap*>(_v) = _t->genericName(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->iD(); break;
        case 9: *reinterpret_cast<QStringMap*>(_v) = _t->icons(); break;
        case 10: *reinterpret_cast<qlonglong*>(_v) = _t->installedTime(); break;
        case 11: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->instances(); break;
        case 12: *reinterpret_cast<qlonglong*>(_v) = _t->lastLaunchedTime(); break;
        case 13: *reinterpret_cast<qlonglong*>(_v) = _t->launchedTimes(); break;
        case 14: *reinterpret_cast<QStringList*>(_v) = _t->mimeTypes(); break;
        case 15: *reinterpret_cast<QStringMap*>(_v) = _t->name(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->noDisplay(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->startupWMClass(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->terminal(); break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->x_CreatedBy(); break;
        case 20: *reinterpret_cast<QString*>(_v) = _t->x_Deepin_CreateBy(); break;
        case 21: *reinterpret_cast<QString*>(_v) = _t->x_Deepin_Vendor(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->x_Flatpak(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->x_linglong(); break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->isOnDesktop(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setAutoStart(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setEnviron(*reinterpret_cast<QString*>(_v)); break;
        case 14: _t->setMimeTypes(*reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ApplicationAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ApplicationAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ApplicationAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int ApplicationAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}
QT_WARNING_POP
