/****************************************************************************
** Meta object code from reading C++ file 'applicationservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/dbus/applicationservice.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'applicationservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18ApplicationServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto ApplicationService::qt_create_metaobjectdata<qt_meta_tag_ZN18ApplicationServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ApplicationService",
        "InterfacesAdded",
        "",
        "QDBusObjectPath",
        "object_path",
        "ObjectInterfaceMap",
        "interfaces",
        "InterfacesRemoved",
        "autostartChanged",
        "noDisplayChanged",
        "isOnDesktopChanged",
        "installedTimeChanged",
        "x_FlatpakChanged",
        "x_linglongChanged",
        "instanceChanged",
        "lastLaunchedTimeChanged",
        "iconsChanged",
        "nameChanged",
        "genericNameChanged",
        "actionNameChanged",
        "actionsChanged",
        "categoriesChanged",
        "MimeTypesChanged",
        "terminalChanged",
        "environChanged",
        "launchedTimesChanged",
        "startupWMClassChanged",
        "xDeepinCreatedByChanged",
        "execsChanged",
        "xCreatedByChanged",
        "desktopSourcePathChanged",
        "Launch",
        "action",
        "fields",
        "QVariantMap",
        "options",
        "GetManagedObjects",
        "ObjectMap",
        "SendToDesktop",
        "RemoveFromDesktop",
        "Categories",
        "Actions",
        "ActionName",
        "PropMap",
        "ID",
        "Name",
        "QStringMap",
        "GenericName",
        "Icons",
        "LastLaunchedTime",
        "LaunchedTimes",
        "Environ",
        "Terminal",
        "StartupWMClass",
        "AutoStart",
        "MimeTypes",
        "Instances",
        "QList<QDBusObjectPath>",
        "X_Flatpak",
        "X_linglong",
        "X_Deepin_Vendor",
        "InstalledTime",
        "isOnDesktop",
        "NoDisplay",
        "X_Deepin_CreateBy",
        "Execs",
        "X_CreatedBy",
        "DesktopSourcePath"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'InterfacesAdded'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const ObjectInterfaceMap &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'InterfacesRemoved'
        QtMocHelpers::SignalData<void(const QDBusObjectPath &, const QStringList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QStringList, 6 },
        }}),
        // Signal 'autostartChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noDisplayChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isOnDesktopChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'installedTimeChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'x_FlatpakChanged'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'x_linglongChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'instanceChanged'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lastLaunchedTimeChanged'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconsChanged'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'genericNameChanged'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'actionNameChanged'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'actionsChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'categoriesChanged'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'MimeTypesChanged'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'terminalChanged'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'environChanged'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'launchedTimesChanged'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startupWMClassChanged'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'xDeepinCreatedByChanged'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'execsChanged'
        QtMocHelpers::SignalData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'xCreatedByChanged'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'desktopSourcePathChanged'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Launch'
        QtMocHelpers::SlotData<QDBusObjectPath(const QString &, const QStringList &, const QVariantMap &)>(31, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 32 }, { QMetaType::QStringList, 33 }, { 0x80000000 | 34, 35 },
        }}),
        // Slot 'GetManagedObjects'
        QtMocHelpers::SlotData<ObjectMap() const>(36, 2, QMC::AccessPublic, 0x80000000 | 37),
        // Slot 'SendToDesktop'
        QtMocHelpers::SlotData<bool() const>(38, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'RemoveFromDesktop'
        QtMocHelpers::SlotData<bool() const>(39, 2, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Categories'
        QtMocHelpers::PropertyData<QStringList>(40, QMetaType::QStringList, QMC::DefaultPropertyFlags, 15),
        // property 'Actions'
        QtMocHelpers::PropertyData<QStringList>(41, QMetaType::QStringList, QMC::DefaultPropertyFlags, 14),
        // property 'ActionName'
        QtMocHelpers::PropertyData<PropMap>(42, 0x80000000 | 43, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 13),
        // property 'ID'
        QtMocHelpers::PropertyData<QString>(44, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'Name'
        QtMocHelpers::PropertyData<QStringMap>(45, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 11),
        // property 'GenericName'
        QtMocHelpers::PropertyData<QStringMap>(47, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 12),
        // property 'Icons'
        QtMocHelpers::PropertyData<QStringMap>(48, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 10),
        // property 'LastLaunchedTime'
        QtMocHelpers::PropertyData<qint64>(49, QMetaType::LongLong, QMC::DefaultPropertyFlags, 9),
        // property 'LaunchedTimes'
        QtMocHelpers::PropertyData<qint64>(50, QMetaType::LongLong, QMC::DefaultPropertyFlags, 19),
        // property 'Environ'
        QtMocHelpers::PropertyData<QString>(51, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'Terminal'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags, 17),
        // property 'StartupWMClass'
        QtMocHelpers::PropertyData<QString>(53, QMetaType::QString, QMC::DefaultPropertyFlags, 20),
        // property 'AutoStart'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'MimeTypes'
        QtMocHelpers::PropertyData<QStringList>(55, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'Instances'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(56, 0x80000000 | 57, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 8),
        // property 'X_Flatpak'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'X_linglong'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'X_Deepin_Vendor'
        QtMocHelpers::PropertyData<QString>(60, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'InstalledTime'
        QtMocHelpers::PropertyData<qint64>(61, QMetaType::LongLong, QMC::DefaultPropertyFlags, 5),
        // property 'isOnDesktop'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'NoDisplay'
        QtMocHelpers::PropertyData<bool>(63, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'X_Deepin_CreateBy'
        QtMocHelpers::PropertyData<QString>(64, QMetaType::QString, QMC::DefaultPropertyFlags, 21),
        // property 'Execs'
        QtMocHelpers::PropertyData<QStringMap>(65, 0x80000000 | 46, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 22),
        // property 'X_CreatedBy'
        QtMocHelpers::PropertyData<QString>(66, QMetaType::QString, QMC::DefaultPropertyFlags, 23),
        // property 'DesktopSourcePath'
        QtMocHelpers::PropertyData<QString>(67, QMetaType::QString, QMC::DefaultPropertyFlags, 24),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ApplicationService, qt_meta_tag_ZN18ApplicationServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ApplicationService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ApplicationServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ApplicationServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18ApplicationServiceE_t>.metaTypes,
    nullptr
} };

void ApplicationService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ApplicationService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->InterfacesAdded((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ObjectInterfaceMap>>(_a[2]))); break;
        case 1: _t->InterfacesRemoved((*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 2: _t->autostartChanged(); break;
        case 3: _t->noDisplayChanged(); break;
        case 4: _t->isOnDesktopChanged(); break;
        case 5: _t->installedTimeChanged(); break;
        case 6: _t->x_FlatpakChanged(); break;
        case 7: _t->x_linglongChanged(); break;
        case 8: _t->instanceChanged(); break;
        case 9: _t->lastLaunchedTimeChanged(); break;
        case 10: _t->iconsChanged(); break;
        case 11: _t->nameChanged(); break;
        case 12: _t->genericNameChanged(); break;
        case 13: _t->actionNameChanged(); break;
        case 14: _t->actionsChanged(); break;
        case 15: _t->categoriesChanged(); break;
        case 16: _t->MimeTypesChanged(); break;
        case 17: _t->terminalChanged(); break;
        case 18: _t->environChanged(); break;
        case 19: _t->launchedTimesChanged(); break;
        case 20: _t->startupWMClassChanged(); break;
        case 21: _t->xDeepinCreatedByChanged(); break;
        case 22: _t->execsChanged(); break;
        case 23: _t->xCreatedByChanged(); break;
        case 24: _t->desktopSourcePathChanged(); break;
        case 25: { QDBusObjectPath _r = _t->Launch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 26: { ObjectMap _r = _t->GetManagedObjects();
            if (_a[0]) *reinterpret_cast<ObjectMap*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->SendToDesktop();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->RemoveFromDesktop();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ObjectInterfaceMap >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)(const QDBusObjectPath & , const ObjectInterfaceMap & )>(_a, &ApplicationService::InterfacesAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)(const QDBusObjectPath & , const QStringList & )>(_a, &ApplicationService::InterfacesRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::autostartChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::noDisplayChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::isOnDesktopChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::installedTimeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::x_FlatpakChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::x_linglongChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::instanceChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::lastLaunchedTimeChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::iconsChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::nameChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::genericNameChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::actionNameChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::actionsChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::categoriesChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::MimeTypesChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::terminalChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::environChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::launchedTimesChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::startupWMClassChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::xDeepinCreatedByChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::execsChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::xCreatedByChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (ApplicationService::*)()>(_a, &ApplicationService::desktopSourcePathChanged, 24))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PropMap >(); break;
        case 14:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        case 22:
        case 6:
        case 5:
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QStringMap >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->categories(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->actions(); break;
        case 2: *reinterpret_cast<PropMap*>(_v) = _t->actionName(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 4: *reinterpret_cast<QStringMap*>(_v) = _t->name(); break;
        case 5: *reinterpret_cast<QStringMap*>(_v) = _t->genericName(); break;
        case 6: *reinterpret_cast<QStringMap*>(_v) = _t->icons(); break;
        case 7: *reinterpret_cast<qint64*>(_v) = _t->lastLaunchedTime(); break;
        case 8: *reinterpret_cast<qint64*>(_v) = _t->launchedTimes(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->environ(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->terminal(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->startupWMClass(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->isAutoStart(); break;
        case 13: *reinterpret_cast<QStringList*>(_v) = _t->mimeTypes(); break;
        case 14: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->instances(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->x_Flatpak(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->x_linglong(); break;
        case 17: *reinterpret_cast<QString*>(_v) = _t->X_Deepin_Vendor(); break;
        case 18: *reinterpret_cast<qint64*>(_v) = _t->installedTime(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->isOnDesktop(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->noDisplay(); break;
        case 21: *reinterpret_cast<QString*>(_v) = _t->X_Deepin_CreateBy(); break;
        case 22: *reinterpret_cast<QStringMap*>(_v) = _t->execs(); break;
        case 23: *reinterpret_cast<QString*>(_v) = _t->X_CreatedBy(); break;
        case 24: *reinterpret_cast<QString*>(_v) = _t->desktopSourcePath(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 9: _t->setEnviron(*reinterpret_cast<QString*>(_v)); break;
        case 12: _t->setAutoStart(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setMimeTypes(*reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ApplicationService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ApplicationService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ApplicationServiceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int ApplicationService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void ApplicationService::InterfacesAdded(const QDBusObjectPath & _t1, const ObjectInterfaceMap & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void ApplicationService::InterfacesRemoved(const QDBusObjectPath & _t1, const QStringList & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void ApplicationService::autostartChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ApplicationService::noDisplayChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ApplicationService::isOnDesktopChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ApplicationService::installedTimeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ApplicationService::x_FlatpakChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void ApplicationService::x_linglongChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void ApplicationService::instanceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void ApplicationService::lastLaunchedTimeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void ApplicationService::iconsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void ApplicationService::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void ApplicationService::genericNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void ApplicationService::actionNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void ApplicationService::actionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void ApplicationService::categoriesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void ApplicationService::MimeTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void ApplicationService::terminalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void ApplicationService::environChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void ApplicationService::launchedTimesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void ApplicationService::startupWMClassChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void ApplicationService::xDeepinCreatedByChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void ApplicationService::execsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void ApplicationService::xCreatedByChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void ApplicationService::desktopSourcePathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}
QT_WARNING_POP
