// SPDX-FileCopyrightText: 2023 - 2026 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef CONFIG_H
#define CONFIG_H

constexpr static auto &ApplicationLaunchHelperBinary =
    u"/usr/lib/deepin/application-manager/app-launch-helper";

constexpr static auto &ApplicationServiceID = u"org.deepin.dde.application-manager";

constexpr static auto &DDEApplicationBin = u"/usr/bin/dde-am";

#endif
