// This file was generated by qtwaylandscanner
// source file is treeland-prelaunch-splash-v2.xml

#include "qwayland-treeland-prelaunch-splash-v2.h"

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {

static inline void *wlRegistryBind(struct ::wl_registry *registry, uint32_t name, const struct ::wl_interface *interface, uint32_t version)
{
    const uint32_t bindOpCode = 0;
    return (void *) wl_proxy_marshal_constructor_versioned((struct wl_proxy *) registry,
    bindOpCode, interface, version, name, interface->name, version, nullptr);
}

    treeland_prelaunch_splash_manager_v2::treeland_prelaunch_splash_manager_v2(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    treeland_prelaunch_splash_manager_v2::treeland_prelaunch_splash_manager_v2(struct ::treeland_prelaunch_splash_manager_v2 *obj)
        : m_treeland_prelaunch_splash_manager_v2(obj)
    {
    }

    treeland_prelaunch_splash_manager_v2::treeland_prelaunch_splash_manager_v2()
        : m_treeland_prelaunch_splash_manager_v2(nullptr)
    {
    }

    treeland_prelaunch_splash_manager_v2::~treeland_prelaunch_splash_manager_v2()
    {
    }

    void treeland_prelaunch_splash_manager_v2::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_treeland_prelaunch_splash_manager_v2 = static_cast<struct ::treeland_prelaunch_splash_manager_v2 *>(wlRegistryBind(registry, id, &treeland_prelaunch_splash_manager_v2_interface, version));
    }

    void treeland_prelaunch_splash_manager_v2::init(struct ::treeland_prelaunch_splash_manager_v2 *obj)
    {
        m_treeland_prelaunch_splash_manager_v2 = obj;
    }

    treeland_prelaunch_splash_manager_v2 *treeland_prelaunch_splash_manager_v2::fromObject(struct ::treeland_prelaunch_splash_manager_v2 *object)
    {
        return static_cast<treeland_prelaunch_splash_manager_v2 *>(treeland_prelaunch_splash_manager_v2_get_user_data(object));
    }

    bool treeland_prelaunch_splash_manager_v2::isInitialized() const
    {
        return m_treeland_prelaunch_splash_manager_v2 != nullptr;
    }

    uint32_t treeland_prelaunch_splash_manager_v2::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_treeland_prelaunch_splash_manager_v2));
    }

    const struct wl_interface *treeland_prelaunch_splash_manager_v2::interface()
    {
        return &::treeland_prelaunch_splash_manager_v2_interface;
    }

    void treeland_prelaunch_splash_manager_v2::destroy()
    {
        ::treeland_prelaunch_splash_manager_v2_destroy(
            m_treeland_prelaunch_splash_manager_v2);
        m_treeland_prelaunch_splash_manager_v2 = nullptr;
    }

    struct ::treeland_prelaunch_splash_v2 *treeland_prelaunch_splash_manager_v2::create_splash(const QString &app_id, const QString &instance_id, const QString &sandbox_engine_name, struct ::wl_buffer *icon_buffer)
    {
        struct ::treeland_prelaunch_splash_v2 * splash = ::treeland_prelaunch_splash_manager_v2_create_splash(
            m_treeland_prelaunch_splash_manager_v2,
            app_id.toUtf8().constData(),
            instance_id.toUtf8().constData(),
            sandbox_engine_name.toUtf8().constData(),
            icon_buffer);
        return splash;
    }

    treeland_prelaunch_splash_v2::treeland_prelaunch_splash_v2(struct ::wl_registry *registry, uint32_t id, int version)
    {
        init(registry, id, version);
    }

    treeland_prelaunch_splash_v2::treeland_prelaunch_splash_v2(struct ::treeland_prelaunch_splash_v2 *obj)
        : m_treeland_prelaunch_splash_v2(obj)
    {
    }

    treeland_prelaunch_splash_v2::treeland_prelaunch_splash_v2()
        : m_treeland_prelaunch_splash_v2(nullptr)
    {
    }

    treeland_prelaunch_splash_v2::~treeland_prelaunch_splash_v2()
    {
    }

    void treeland_prelaunch_splash_v2::init(struct ::wl_registry *registry, uint32_t id, int version)
    {
        m_treeland_prelaunch_splash_v2 = static_cast<struct ::treeland_prelaunch_splash_v2 *>(wlRegistryBind(registry, id, &treeland_prelaunch_splash_v2_interface, version));
    }

    void treeland_prelaunch_splash_v2::init(struct ::treeland_prelaunch_splash_v2 *obj)
    {
        m_treeland_prelaunch_splash_v2 = obj;
    }

    treeland_prelaunch_splash_v2 *treeland_prelaunch_splash_v2::fromObject(struct ::treeland_prelaunch_splash_v2 *object)
    {
        return static_cast<treeland_prelaunch_splash_v2 *>(treeland_prelaunch_splash_v2_get_user_data(object));
    }

    bool treeland_prelaunch_splash_v2::isInitialized() const
    {
        return m_treeland_prelaunch_splash_v2 != nullptr;
    }

    uint32_t treeland_prelaunch_splash_v2::version() const
    {
        return wl_proxy_get_version(reinterpret_cast<wl_proxy*>(m_treeland_prelaunch_splash_v2));
    }

    const struct wl_interface *treeland_prelaunch_splash_v2::interface()
    {
        return &::treeland_prelaunch_splash_v2_interface;
    }

    void treeland_prelaunch_splash_v2::destroy()
    {
        ::treeland_prelaunch_splash_v2_destroy(
            m_treeland_prelaunch_splash_v2);
        m_treeland_prelaunch_splash_v2 = nullptr;
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
