/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.MimeManager1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef MIMEMANAGER1ADAPTOR_H
#define MIMEMANAGER1ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "dbus/mimemanager1service.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.desktopspec.MimeManager1
 */
class MimeManager1Adaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.desktopspec.MimeManager1")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.desktopspec.MimeManager1\">\n"
"    <method name=\"queryDefaultApplication\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"content\"/>\n"
"      <arg direction=\"out\" type=\"s\" name=\"mimeType\"/>\n"
"      <arg direction=\"out\" type=\"o\" name=\"application\"/>\n"
"      <annotation value=\"content can be absolute path of a file or a mime type.\" name=\"org.freedesktop.DBus.Description\"/>\n"
"    </method>\n"
"    <method name=\"setDefaultApplication\">\n"
"      <arg direction=\"in\" type=\"a{ss}\" name=\"defaultApps\"/>\n"
"      <annotation value=\"QStringMap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"    </method>\n"
"    <method name=\"unsetDefaultApplication\">\n"
"      <arg direction=\"in\" type=\"as\" name=\"mimeTypes\"/>\n"
"    </method>\n"
"    <method name=\"listApplications\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"mimeType\"/>\n"
"      <arg direction=\"out\" type=\"a{oa{sa{sv}}}\" name=\"applications_and_properties\"/>\n"
"      <annotation value=\"ObjectMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <signal name=\"MimeInfoReloaded\">\n"
"      <annotation value=\"Signal emitted when the mime infos are reloaded.\" name=\"org.freedesktop.DBus.Description\"/>\n"
"    </signal>\n"
"  </interface>\n"
        "")
public:
    MimeManager1Adaptor(MimeManager1Service *parent);
    ~MimeManager1Adaptor() override;

    inline MimeManager1Service *parent() const
    { return static_cast<MimeManager1Service *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    ObjectMap listApplications(const QString &mimeType);
    QString queryDefaultApplication(const QString &content, QDBusObjectPath &application);
    void setDefaultApplication(const QStringMap &defaultApps);
    void unsetDefaultApplication(const QStringList &mimeTypes);
Q_SIGNALS: // SIGNALS
    void MimeInfoReloaded();
};

#endif
