/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.ObjectManager1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef AMOBJECTMANAGER1ADAPTOR_H
#define AMOBJECTMANAGER1ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "dbus/applicationmanager1service.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.desktopspec.DBus.ObjectManager
 */
class AMObjectManagerAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.desktopspec.DBus.ObjectManager")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.desktopspec.DBus.ObjectManager\">\n"
"    <method name=\"GetManagedObjects\">\n"
"      <arg direction=\"out\" type=\"a{oa{sa{sv}}}\" name=\"objpath_interfaces_and_properties\"/>\n"
"      <annotation value=\"ObjectMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <signal name=\"InterfacesAdded\">\n"
"      <arg type=\"o\" name=\"object_path\"/>\n"
"      <arg type=\"a{sa{sv}}\" name=\"interfaces_and_properties\"/>\n"
"      <annotation value=\"ObjectInterfaceMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out1\"/>\n"
"    </signal>\n"
"    <signal name=\"InterfacesRemoved\">\n"
"      <arg type=\"o\" name=\"object_path\"/>\n"
"      <arg type=\"as\" name=\"interfaces\"/>\n"
"    </signal>\n"
"  </interface>\n"
        "")
public:
    AMObjectManagerAdaptor(ApplicationManager1Service *parent);
    ~AMObjectManagerAdaptor() override;

    inline ApplicationManager1Service *parent() const
    { return static_cast<ApplicationManager1Service *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    ObjectMap GetManagedObjects();
Q_SIGNALS: // SIGNALS
    void InterfacesAdded(const QDBusObjectPath &object_path, ObjectInterfaceMap interfaces_and_properties);
    void InterfacesRemoved(const QDBusObjectPath &object_path, const QStringList &interfaces);
};

#endif
