/****************************************************************************
** Meta object code from reading C++ file 'instanceadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "instanceadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'instanceadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15InstanceAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto InstanceAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN15InstanceAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "InstanceAdaptor",
        "D-Bus Interface",
        "org.desktopspec.ApplicationManager1.Instance",
        "D-Bus Introspection",
        "  <interface name=\"org.desktopspec.ApplicationManager1.Instance\""
        ">\n    <property access=\"read\" type=\"o\" name=\"Application\">\n"
        "      <annotation value=\"Object path of the Application.         "
        "                That DBus object will impelement                  "
        "      org.desktopspec.ApplicationManager1.Application.            "
        "            NOTE:                        If the application is uni"
        "nstalled, this object path                        will be set to `"
        "/`.\" name=\"org.freedesktop.DBus.Description\"/>\n    </property>"
        "\n    <property access=\"read\" type=\"o\" name=\"SystemdUnitPath\""
        ">\n      <annotation value=\"The systemd unit object path of this "
        "instance                        under systemd user daemon.        "
        "                Some other DE compoments can use this path to     "
        "                   use the cgroup interface provided by systemd.  "
        "                      NOTE:                        This property M"
        "IGHT be empty                        if this instance is not manag"
        "ed by systemd.\" name=\"org.freedesktop.DBus.Description\"/>\n    "
        "</property>\n    <property access=\"read\" type=\"s\" name=\"Launc"
        "her\">\n      <annotation value=\"This property indicates which Ap"
        "plication                        launcher started this instance.\""
        " name=\"org.freedesktop.DBus.Description\"/>\n    </property>\n   "
        " <property access=\"read\" type=\"s\" name=\"LaunchType\">\n      "
        "<annotation value=\"The launch source name of this instance,      "
        "                  e.g. 'dde-launchpad', 'dde-shell', 'autostart'. "
        "                       Defaults to 'unknown' if not specified.\" n"
        "ame=\"org.freedesktop.DBus.Description\"/>\n    </property>\n    <"
        "property access=\"read\" type=\"b\" name=\"Orphaned\">\n      <ann"
        "otation value=\"This property indicates that the application      "
        "                  to which the instance belonged has been removed."
        "\" name=\"org.freedesktop.DBus.Description\"/>\n    </property>\n "
        "   <method name=\"KillAll\">\n      <arg direction=\"in\" type=\"i"
        "\" name=\"signal\"/>\n      <annotation value=\"Force kill this in"
        "stance.                                 ATTENTION: All processes w"
        "hich launched by                                  this instance wi"
        "ll be killed.\" name=\"org.freedesktop.DBus.Description\"/>\n    <"
        "/method>\n  </interface>\n",
        "KillAll",
        "",
        "signal",
        "Application",
        "QDBusObjectPath",
        "LaunchType",
        "Launcher",
        "Orphaned",
        "SystemdUnitPath"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'KillAll'
        QtMocHelpers::SlotData<void(int)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Application'
        QtMocHelpers::PropertyData<QDBusObjectPath>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'LaunchType'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Launcher'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'Orphaned'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'SystemdUnitPath'
        QtMocHelpers::PropertyData<QDBusObjectPath>(13, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<InstanceAdaptor, qt_meta_tag_ZN15InstanceAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject InstanceAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15InstanceAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15InstanceAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15InstanceAdaptorE_t>.metaTypes,
    nullptr
} };

void InstanceAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<InstanceAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->KillAll((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->application(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->launchType(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->launcher(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->orphaned(); break;
        case 4: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->systemdUnitPath(); break;
        default: break;
        }
    }
}

const QMetaObject *InstanceAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InstanceAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15InstanceAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int InstanceAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
