/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.MimeManager1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "mimemanager1adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class MimeManager1Adaptor
 */

MimeManager1Adaptor::MimeManager1Adaptor(MimeManager1Service *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

MimeManager1Adaptor::~MimeManager1Adaptor()
{
    // destructor
}

ObjectMap MimeManager1Adaptor::listApplications(const QString &mimeType)
{
    // handle method call org.desktopspec.MimeManager1.listApplications
    return parent()->listApplications(mimeType);
}

QString MimeManager1Adaptor::queryDefaultApplication(const QString &content, QDBusObjectPath &application)
{
    // handle method call org.desktopspec.MimeManager1.queryDefaultApplication
    return parent()->queryDefaultApplication(content, application);
}

void MimeManager1Adaptor::setDefaultApplication(const QStringMap &defaultApps)
{
    // handle method call org.desktopspec.MimeManager1.setDefaultApplication
    parent()->setDefaultApplication(defaultApps);
}

void MimeManager1Adaptor::unsetDefaultApplication(const QStringList &mimeTypes)
{
    // handle method call org.desktopspec.MimeManager1.unsetDefaultApplication
    parent()->unsetDefaultApplication(mimeTypes);
}


#include "moc_mimemanager1adaptor.cpp"
