/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.ObjectManager1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "AMobjectmanager1adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class AMObjectManagerAdaptor
 */

AMObjectManagerAdaptor::AMObjectManagerAdaptor(ApplicationManager1Service *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

AMObjectManagerAdaptor::~AMObjectManagerAdaptor()
{
    // destructor
}

ObjectMap AMObjectManagerAdaptor::GetManagedObjects()
{
    // handle method call org.desktopspec.DBus.ObjectManager.GetManagedObjects
    return parent()->GetManagedObjects();
}


#include "moc_AMobjectmanager1adaptor.cpp"
