/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.desktopspec.JobManager1.Job.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "jobadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class JobAdaptor
 */

JobAdaptor::JobAdaptor(JobService *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

JobAdaptor::~JobAdaptor()
{
    // destructor
}

QString JobAdaptor::status() const
{
    // get the value of property Status
    return qvariant_cast< QString >(parent()->property("Status"));
}

void JobAdaptor::Cancel()
{
    // handle method call org.desktopspec.JobManager1.Job.Cancel
    parent()->Cancel();
}

void JobAdaptor::Resume()
{
    // handle method call org.desktopspec.JobManager1.Job.Resume
    parent()->Resume();
}

void JobAdaptor::Suspend()
{
    // handle method call org.desktopspec.JobManager1.Job.Suspend
    parent()->Suspend();
}


#include "moc_jobadaptor.cpp"
